#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <nrcvax.h>
#include "cdmod.h"


void ModifyBondRecords( struct atom_list *alist  )
{
    struct atom_list *current;
    struct command   *cmd;

    int hit, answer;
    char *name;

    do {
       hit = 0;
       name = SelectName( alist );
       if( NULL == name ) {
           fputs( "Bad atom name.\n", stderr );
           continue;
       }

       cmd = SelectAction();
       if( NULL == cmd->action ) {
           fputs( "Bad command.\n", stderr );
           continue;
       }
 
       for( current = alist;
            current->next != NULL;
            current = current->next ) {
              size_t len = strlen( name );
              if( 0 == strncmp( name,
                               current->atom_record->params.at_name, len) ) {
                    hit = 1;
                    cmd->action(current->atom_record );
                    break;
              }
       }
       if( 0 == hit ) 
           fprintf( stderr, "Sorry, atom \"%s\" not found.\n", name );
    } while( (answer = YesNo("Do you want to choose another atom?")) == 'y'); 

    return;
}

char *SelectName( struct atom_list *list )
{
    struct atom_list *current;
    int i_elem;
    size_t len;

    char answer[LINE_LEN],
         *ret_val,
         *ptr;

    i_elem = 0;
    for( current = list;
         current->next != NULL;
         current = current->next ) {
                   i_elem++;
                   fprintf( stdout, "%4d %.6s\t", current->atom_num,
                            current->atom_record->params.at_name );
 
                   if(  0 == i_elem % N_COL ) fputs( "\n", stdout );
    }
     
    fputs( "\nEnter name of atom you want: ", stdout );
    fflush( stdout );
    if( NULL == fgets(answer, sizeof(answer), stdin) ) {
        fprintf( stderr, "fgets() failed -- %s:%d Exiting.\n", 
                          __FILE__, __LINE__ );
        exit( EXIT_FAILURE );
    }
   if( NULL != (ptr = strchr(answer, '\n')) ) *ptr = '\0';
   ptr = ConvertNames( answer );
   if( !ptr )
       return NULL;

   len = strlen( answer );
   ret_val = malloc( len + 1 );
   if( !ret_val ) return NULL;

   strncpy( ret_val, answer, len );
   ret_val[len] = '\0';

   return ret_val;
  
}
 
struct command *SelectAction( void )
{

/* make this struct and the pointer, 'cptr' static, so that caller
 * can access results.
 */
    static struct command comm[] = {
                  { "reorder", OrderBondRecords, 'r' }, 
                  { "delete" , DeleteBondRecords, 'd', },
                  { ""      , NULL, '\0'}
                  };

    static struct command *cptr = comm;

    int c, ans;

    if( cptr != comm ) cptr = comm;

    fputs( "Modify Bond Record Menu:\n", stdout );
    while( 0 != strcmp(cptr->action_name, "") )  {
      fprintf( stdout, "%s\t\t%c\n", 
                       cptr->action_name, cptr->code );
      cptr++;
    }

    fputs( "Command: ", stdout );
    fflush( stdout );
    ans = getchar();
    while( (c = getchar()) != '\n' && c != EOF );

    cptr = comm;
    while( NULL != cptr->action) {
      if( ans == cptr->code ) break;
      cptr++;
    }

    return cptr;
}
