/* Header file for C programs accessing NRCVAX files. 
 * Written by:
 * Paul D. Boyle
 * Dept. of Chemistry
 * North Carolina State University
 * boyle@laue.chem.ncsu.edu
 */
#ifndef NRCVAX_H
#define NRCVAX_H
 
/* For programs requiring the .re file */

typedef struct {
        float beta;
        int h;
        int k;
        int l;
        float epsilon;
        float Fo;
        float sigma_Fo;
        float Ac;
        float Bc;
        float Ac_dsp;
        float Bc_dsp;
        float sintl;
        float f1;
        float f2;
        float f3;
        float f4;
        float f5;
        float f6;
        float f7;
        float f8;
        float Fo_; /* appended _ denotes Friedel pair */
        float sigma_Fo_;
        int obs_unobs;
        float f9;
        float f10;
        float f11;
        float f12;
        float Fo2;  /* Denotes Fo**2 */
        float sigma_Fo2;
        float Fo2_;
        float sigma_Fo2_;
        float f_disp;
        } RE_Record;

/* For programs requiring the access to the .cd file. */
/* typedef some structs needed to build the Record type structs */

typedef struct {
        float a;
        float b;
        float c;
        float cos_alpha;
        float cos_beta;
        float cos_gamma;
        float sin_alpha;
        float sin_beta;
        float sin_gamma;
        float alpha;
        float beta;
        float gamma;
        } NRCCell;

typedef struct {
        int s[9];
        int t[3];
        } Rt_Matrix;

typedef struct {
        char atsymb[4];
        float atcnt;
        int iatno;
        float atwt;
        float absco;
        float dfp;
        float dfdp;
        float sctcof[9];
        } AtomTypeInfo;

#define ATOM_NAME_LEN  8

typedef struct {
        char at_name[ATOM_NAME_LEN];
        char site_sym_1[8];
        int sym_op_flag;
        float occ;
        float xyz[3];
        float U[6];
        float sigma_occ;
        float sigma_xyz[3];
        float sigma_U[6];
        int U_flag;
        int scatf_seq;
        unsigned long refine_flag;
        int ibx;
        int ianom;
        int mult;
        } Atom;


typedef struct {
        char name[ATOM_NAME_LEN];
        float dist;
        } BondRecord;

/* Now define the Record typedef structs */

typedef struct {
        char Title[48];
        char Job[48];
        char var25[4];
        char var26[4];
        int date;
        NRCCell direct;
        NRCCell recip;
        float direct_vol;
        float recip_vol;
        float S_matrx[6];
        float sigma_a;
        float sigma_b;
        float sigma_c;
        float sigma_cosalp;
        float sigma_cosbet;
        float sigma_cosgam;
        int h_max;
        int k_max;
        int l_max;
        int nrefl;
        float Fo_max;
        float sintl_max;
        int nscale;
        float avg_eee;
        float avg_eeo;
        float avg_eoe;
        float avg_eoo;
        float avg_oee;
        float avg_oeo;
        float avg_ooe;
        float avg_ooo;
        char down[4];
        char across[4];
        char section[4];
        float map_start_x;
        float map_start_y;
        float map_start_z;
        float map_end_x;
        float map_end_y;
        float map_end_z;
        float sigma_alp;
        float sigma_bet;
        float sigma_gam;
        int val_ref_meas;
        int uniq_ref_meas;
        int signf_ref;
        float sigma_level;
        float del_i_sig_i;
        char var98[4];
        char var99[4];
        char var100[4]; 
        } Record_1;

typedef struct {
        char sgsymb[40];
        int laue;
        int latcen;
        int icen;
        int nsym;
        int npolnx;
        int ncv;
        float cen[12];
        Rt_Matrix rt1;
        Rt_Matrix rt2;
        Rt_Matrix rt3;
        Rt_Matrix rt4;
        Rt_Matrix rt5;
        Rt_Matrix rt6;
        } Record_2;

typedef struct {
        Rt_Matrix rt1;
        Rt_Matrix rt2;
        Rt_Matrix rt3;
        Rt_Matrix rt4;
        Rt_Matrix rt5;
        Rt_Matrix rt6;
        Rt_Matrix rt7;
        Rt_Matrix rt8;
        char var97[4];
        char var98[4];
        char var99[4];
        char var100[4];
        } Record_3;

typedef struct {
        Rt_Matrix rt1;
        Rt_Matrix rt2;
        Rt_Matrix rt3;
        Rt_Matrix rt4;
        Rt_Matrix rt5;
        Rt_Matrix rt6;
        Rt_Matrix rt7;
        Rt_Matrix rt8;
        char var97[4];
        char var98[4];
        char var99[4];
        char var100[4];
        } Record_4;

typedef struct {
        Rt_Matrix rt1;
        Rt_Matrix rt2;
        AtomTypeInfo atom_info_3[4];
        char var89[4];
        char var90[4];
        char var91[4];
        char var92[4];
        char var93[4];
        char var94[4];
        char var95[4];
        char var96[4];
        char var97[4];
        char var98[4];
        char var99[4];
        char var100[4];
        } Record_5;

typedef struct {
        float lambda;
        int ntyps;
        float sigma1;
        float sigma2;
        float sigma3;
        float e000;
        float eps;
        float dcal;
        float abslin;
        AtomTypeInfo atom_info_1[5];
        char var90[4];
        char var91[4];
        char var92[4];
        char var93[4];
        char var94[4];
        char var95[4];
        char var96[4];
        char var97[4];
        char var98[4];
        char var99[4];
        char var100[4];
        } Record_6;

typedef struct {
        AtomTypeInfo atom_info_2[3];
        float atom_radius_1;
        float atom_radius_2;
        float atom_radius_3;
        float atom_radius_4;
        float atom_radius_5;
        float atom_radius_6;
        float atom_radius_7;
        float atom_radius_8;
        int isym;
        int link;
        int sym_matrx[9];
        float sym_vector[3];
        char link_name[8];
        int sym_apply_flag;
        int start_range;
        int end_range;
        float atom_radius_9;
        float atom_radius_10;
        float atom_radius_11;
        float atom_radius_12;
        char var80[4];
        char var81[4];
        char var82[4];
        char var83[4];
        char var84[4];
        char var85[4];
        char var86[4];
        char var87[4];
        char var88[4];
        char var89[4];
        char var90[4];
        char var91[4];
        char var92[4];
        char var93[4];
        char var94[4];
        char var95[4];
        char var96[4];
        char var97[4];
        char var98[4];
        char var99[4];
        char var100[4];
        } Record_7;

typedef struct {
        float cut_off;
        float wilson_b;
        float Fo_scale;
        float ext_coeff;
        int refine_flag;
        float prec;
        int weight_flag;
        int n_variables;
        float sigma_scale;
        float sigma_ext_coeff;
        int n_sym_relations;
        float hughes[5];
        float sym_relations[40];
        float unext;
        float rf_sig;
        float rw_sig;
        float rf_all;
        float rw_all;
        float gof;
        int n_ref_incl;
        int n_ref_all;
        int n_atom_incl;
        int n_var_incl;
        int min_e_den;
        int max_e_den;
        int map_type;
        float max_shift_sig;
        float min_trans;
        float max_trans;
        float dcalc;
        int n_ref_cell;
        float min_2t_cell;
        float max_2t_cell;
        float cryst_dim[3];
        int Z;
        int n_pos_sig;
        int n_pos_insig;
        int n_neg_sig;
        int n_neg_insig;
        float chirality;
        float sigma_chirality;
        char var87[4];
        char var88[4];
        char var89[4];
        char var90[4];
        char var91[4];
        char var92[4];
        char var93[4];
        char var94[4];
        char var95[4];
        char var96[4];
        char var97[4];
        char var98[4];
        char var99[4];
        char var100[4];
        } Record_8;

#define N_BOND_RECORDS  8

typedef struct {
        Atom params;
        Atom old_params;
        float covar_xx;
        float covar_xy;
        float covar_xz;
        float covar_yy;
        float covar_yz;
        float covar_zz;
        int n_bonds;
        BondRecord bond_record[N_BOND_RECORDS];
        char var94[4];
        char var95[4];
        char var96[4];
        char var97[4];
        char var98[4];
        char var99[4];
        char var100[4];
        } AtomRecord;

typedef struct {
        char blanks[400];
        } CD_End;

/* Create an enum list and a MACRO to test for the setting of
 * refinement flag bits in the refine_flag member of the Atom
 * structure. Thanks to Lawrence Kirby (fred@genesis.demon.co.uk)
 * for the suggestion on comp.lang.c.
 */

/* Usage:
 * The macro returns 0 if the parameter is allowed to or is set to refine
 * and != 0 if it is *not* allowed or not set to refine.  It would have been
 * cooler if the bit definitions were the other way (i.e. bit set ==  1 means
 * parameter gets refined, but c'est la vie!
 *
 * Example:
 *
 * unsigned long flags;
 * AtomRecord *atom_rec;
 * ...
 * flags = atom_rec->params.refine_flag;
 * IREF_BIT( flags, REFINE_Y );
 *
 * This construct tests to see if the 9th bit (counting from 0 or 10th bit
 *  counting from 1) is set. 
 */

/* ENUMS */

enum {
     REFINE_U23, REFINE_U13, REFINE_U12,
     REFINE_U33, REFINE_U22, REFINE_U11,
     REFINE_Z, ALLOW_Z,
     REFINE_Y, ALLOW_Y,
     REFINE_X, ALLOW_X,
     REFINE_OCC
     };

/* MACROS */

#define IREF_BIT( value, bitpos ) (value & (1 << bitpos) )

/* If I ever find out that there is an endian dependency, uncomment
 * the stuff below (and comment out the macro defined above).
 * For your reading pleasure Andy Robb (ajrobb@pavillion.co.uk) points
 * out:
 * #define LITTLE_ENDIAN 1234
 * #define BIG_ENDIAN 4321 
 * #define DEC_VAX 3412  (PDP has some endian as DEC VAX)
 */

/*
#ifdef BIG_ENDIAN 
#define IREF_BIT( value, bitpos ) (value & (1 << (31 - bitpos)) )
#else
#define IREF_BIT( value, bitpos ) (value & (1 << bitpos) )
#endif
*/
#endif     
