/* Order Bond Records.  This function orders the bond records
 * for a given atom to a user specified sequence.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <nrcvax.h>
#include "cdmod.h"

void *OrderBondRecords( AtomRecord *record )
{
    int i, j;
    int n_bonds;

    char atom_name[ATOM_NAME_LEN],
         line[LINE_LEN],
         *ptr;

    AtomNameArray ordered_names;

    BondRecord  tmp_bond[N_BOND_RECORDS];

    strncpy( atom_name, record->params.at_name, ATOM_NAME_LEN );
    atom_name[ATOM_NAME_LEN - 1] = '\0';

    n_bonds = record->n_bonds < N_BOND_RECORDS ?
                                record->n_bonds : N_BOND_RECORDS;

    fprintf( stdout, "List of bond records for atom %s\n", atom_name );
                      
    for( i = 0; i < n_bonds; i++ )
         fprintf( stdout, "%.8s %7.3f\n",  
                                         record->bond_record[i].name,
                                         record->bond_record[i].dist );
    fputs( "\n", stdout );


/* get the desired order from the user */
    fputs( "Enter atom names in the desired sequence (CR=keep): ", stdout );
    fflush( stdout );

    if( NULL == fgets(line, sizeof(line), stdin) ) {
        fprintf( stderr, "fgets() failed -- %s:%d\n", __FILE__, __LINE__ );
        exit( EXIT_FAILURE );
    }
    if( NULL != (ptr = strchr(line, '\n')) ) *ptr = '\0';

    if( 0 == strncmp(line, "", 1) ) {
        fputs( "Keeping bond records in original order.\n", stdout );
        return NULL;
    }

    ptr = ConvertNames( line );
    if( NULL == ptr ) {
        fprintf( stderr, "Input problem with line: %s\n", line );
        return NULL;
    }
    ordered_names = SplitAtomNameList( line );
    if( NULL == ordered_names ) {
        fprintf( stderr, "Problem parsing data in: %s\n", line );
        return NULL;
    }
        

/* write the bond records into the desired order using a temporary 
 * array, and copy back into  'record's bond_record array.
 */
    for( i = 0; i < n_bonds; i++ ) {
         for( j = 0; j < n_bonds; j++ ) {
              if( 0 == memcmp( ordered_names[i],
                               record->bond_record[j].name, ATOM_NAME_LEN) )
                  tmp_bond[i] = record->bond_record[j];
         }
    }

    memcpy( record->bond_record, tmp_bond, n_bonds * sizeof(BondRecord) );

    free( ordered_names );
    return record;
}
